import { DOCUMENT } from "@angular/common";
import { ChangeDetectionStrategy, ChangeDetectorRef, Component, ComponentFactoryResolver, ContentChildren, ElementRef, EventEmitter, HostBinding, Inject, Input, IterableDiffers, NgZone, Output, QueryList, TemplateRef, ViewChild, ViewChildren, ViewContainerRef } from "@angular/core";
import { of } from "rxjs/observable/of";
import { debounceTime, delay, merge, repeat, take, takeUntil } from "rxjs/operators";
import { Subject } from "rxjs/Subject";
import { IgxSelectionAPIService } from "../core/selection";
import { cloneArray } from "../core/utils";
import { DataType } from "../data-operations/data-util";
import { FilteringLogic } from "../data-operations/filtering-expression.interface";
import { SortingDirection } from "../data-operations/sorting-expression.interface";
import { IgxForOfDirective } from "../directives/for-of/for_of.directive";
import { IgxCheckboxComponent } from "./../checkbox/checkbox.component";
import { IgxGridAPIService } from "./api.service";
import { IgxColumnComponent } from "./column.component";
import { IgxGridRowComponent } from "./row.component";
var NEXT_ID = 0;
var DEBOUNCE_TIME = 16;
var DEFAULT_SUMMARY_HEIGHT = 36.36;
var MINIMUM_COLUMN_WIDTH = 136;
/**
 * **Ignite UI for Angular Grid** -
 * [Documentation](https://www.infragistics.com/products/ignite-ui-angular/angular/components/grid.html)
 *
 * The Ignite UI Grid is used for presenting and manipulating tabular data in the simplest way possible.  Once data
 * has been bound, it can be manipulated through filtering, sorting & editing operations.
 *
 * Example:
 * ```html
 * <igx-grid [data]="employeeData" autoGenerate="false">
 *   <igx-column field="first" header="First Name"></igx-column>
 *   <igx-column field="last" header="Last Name"></igx-column>
 *   <igx-column field="role" header="Role"></igx-column>
 * </igx-grid>
 * ```
 */
var IgxGridComponent = /** @class */ (function () {
    function IgxGridComponent(gridAPI, selectionAPI, elementRef, zone, document, cdr, resolver, differs, viewRef) {
        var _this = this;
        this.gridAPI = gridAPI;
        this.selectionAPI = selectionAPI;
        this.elementRef = elementRef;
        this.zone = zone;
        this.document = document;
        this.cdr = cdr;
        this.resolver = resolver;
        this.differs = differs;
        this.viewRef = viewRef;
        this.data = [];
        this.autoGenerate = false;
        this.id = "igx-grid-" + NEXT_ID++;
        this.filteringLogic = FilteringLogic.And;
        this.evenRowCSS = "";
        this.oddRowCSS = "";
        this.rowHeight = 50;
        this.columnWidth = null;
        this.onCellClick = new EventEmitter();
        this.onSelection = new EventEmitter();
        this.onRowSelectionChange = new EventEmitter();
        this.onColumnPinning = new EventEmitter();
        /**
             * An @Output property emitting an event when cell or row editing has been performed in the grid.
             * On cell editing, both cell and row objects in the event arguments are defined for the corresponding
             * cell that is being edited and the row the cell belongs to.
             * On row editing, only the row object is defined, for the row that is being edited.
             * The cell object is null on row editing.
             */
        this.onEditDone = new EventEmitter();
        this.onColumnInit = new EventEmitter();
        this.onSortingDone = new EventEmitter();
        this.onFilteringDone = new EventEmitter();
        this.onPagingDone = new EventEmitter();
        this.onRowAdded = new EventEmitter();
        this.onRowDeleted = new EventEmitter();
        this.onDataPreLoad = new EventEmitter();
        this.onColumnResized = new EventEmitter();
        this.onContextMenu = new EventEmitter();
        this.tabindex = 0;
        this.hostClass = "igx-grid";
        this.hostRole = "grid";
        this.eventBus = new Subject();
        this.allRowsSelected = false;
        this.destroy$ = new Subject();
        this._perPage = 15;
        this._page = 0;
        this._paging = false;
        this._rowSelection = false;
        this._pipeTrigger = 0;
        this._columns = [];
        this._pinnedColumns = [];
        this._unpinnedColumns = [];
        this._filteringLogic = FilteringLogic.And;
        this._filteringExpressions = [];
        this._sortingExpressions = [];
        this._filteredData = null;
        this._height = "100%";
        this._width = "100%";
        this.resizeHandler = function () {
            _this.calculateGridSizes();
            _this.zone.run(function () { return _this.markForCheck(); });
        };
    }
    Object.defineProperty(IgxGridComponent.prototype, "filteringExpressions", {
        get: function () {
            return this._filteringExpressions;
        },
        set: function (value) {
            this._filteringExpressions = cloneArray(value);
            this.cdr.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "filteredData", {
        get: function () {
            return this._filteredData;
        },
        set: function (value) {
            this._filteredData = value;
            if (this.rowSelectable) {
                this.updateHeaderChecboxStatusOnFilter(this._filteredData);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "paging", {
        get: function () {
            return this._paging;
        },
        set: function (value) {
            this._paging = value;
            this._pipeTrigger++;
            this.cdr.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "page", {
        get: function () {
            return this._page;
        },
        set: function (val) {
            if (val < 0) {
                return;
            }
            this.onPagingDone.emit({ previous: this._page, current: val });
            this._page = val;
            this.cdr.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "perPage", {
        get: function () {
            return this._perPage;
        },
        set: function (val) {
            if (val < 0) {
                return;
            }
            this._perPage = val;
            this.page = 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "rowSelectable", {
        get: function () {
            return this._rowSelection;
        },
        set: function (val) {
            this._rowSelection = val;
            if (this.gridAPI.get(this.id)) {
                // should selection persist?
                this.allRowsSelected = false;
                this.deselectAllRows();
                this.markForCheck();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "height", {
        get: function () {
            return this._height;
        },
        set: function (value) {
            var _this = this;
            if (this._height !== value) {
                this._height = value;
                requestAnimationFrame(function () {
                    _this.calculateGridHeight();
                    _this.cdr.markForCheck();
                });
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "width", {
        get: function () {
            return this._width;
        },
        set: function (value) {
            var _this = this;
            if (this._width !== value) {
                this._width = value;
                requestAnimationFrame(function () {
                    _this.calculateGridWidth();
                    _this.cdr.markForCheck();
                });
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "headerWidth", {
        get: function () {
            return parseInt(this._width, 10) - 17;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "pipeTrigger", {
        get: function () {
            return this._pipeTrigger;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "sortingExpressions", {
        get: function () {
            return this._sortingExpressions;
        },
        set: function (value) {
            this._sortingExpressions = cloneArray(value);
            this.cdr.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "virtualizationState", {
        get: function () {
            return this.verticalScrollContainer.state;
        },
        set: function (state) {
            this.verticalScrollContainer.state = state;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "totalItemCount", {
        get: function () {
            return this.verticalScrollContainer.totalItemCount;
        },
        set: function (count) {
            this.verticalScrollContainer.totalItemCount = count;
            this.cdr.detectChanges();
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype.ngOnInit = function () {
        this.gridAPI.register(this);
        this.setEventBusSubscription();
        this.setVerticalScrollSubscription();
        this.columnListDiffer = this.differs.find([]).create(null);
        this.calcWidth = this._width && this._width.indexOf("%") === -1 ? parseInt(this._width, 10) : 0;
        this.calcHeight = 0;
        this.calcRowCheckboxWidth = 0;
    };
    IgxGridComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        if (this.autoGenerate) {
            this.autogenerateColumns();
        }
        this.initColumns(this.columnList, function (col) { return _this.onColumnInit.emit(col); });
        this.columnListDiffer.diff(this.columnList);
        this.clearSummaryCache();
        this.tfootHeight = this.calcMaxSummaryHeight();
        this._derivePossibleHeight();
        this.markForCheck();
        this.columnList.changes
            .pipe(takeUntil(this.destroy$))
            .subscribe(function (change) {
            var diff = _this.columnListDiffer.diff(change);
            if (diff) {
                _this.initColumns(_this.columnList);
                diff.forEachAddedItem(function (record) {
                    _this.clearSummaryCache();
                    _this.calculateGridSizes();
                    _this.onColumnInit.emit(record.item);
                });
                diff.forEachRemovedItem(function (record) {
                    // Recalculate Summaries
                    // Recalculate Summaries
                    _this.clearSummaryCache();
                    _this.calculateGridSizes();
                    // Clear Filtering
                    // Clear Filtering
                    _this.gridAPI.clear_filter(_this.id, record.item.field);
                    // Clear Sorting
                    // Clear Sorting
                    _this.gridAPI.clear_sort(_this.id, record.item.field);
                });
            }
            _this.markForCheck();
        });
    };
    IgxGridComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.zone.runOutsideAngular(function () {
            _this.document.defaultView.addEventListener("resize", _this.resizeHandler);
        });
        this._derivePossibleWidth();
        this.calculateGridSizes();
    };
    IgxGridComponent.prototype.ngOnDestroy = function () {
        var _this = this;
        this.zone.runOutsideAngular(function () { return _this.document.defaultView.removeEventListener("resize", _this.resizeHandler); });
        this.destroy$.next(true);
        this.destroy$.complete();
    };
    IgxGridComponent.prototype.dataLoading = function (event) {
        this.onDataPreLoad.emit(event);
    };
    Object.defineProperty(IgxGridComponent.prototype, "nativeElement", {
        get: function () {
            return this.elementRef.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "calcResizerHeight", {
        get: function () {
            if (this.hasSummarizedColumns) {
                return this.theadRow.nativeElement.clientHeight + this.tbody.nativeElement.clientHeight +
                    this.tfoot.nativeElement.clientHeight;
            }
            return this.theadRow.nativeElement.clientHeight + this.tbody.nativeElement.clientHeight;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "calcPinnedContainerMaxWidth", {
        get: function () {
            return (this.calcWidth * 80) / 100;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "unpinnedAreaMinWidth", {
        get: function () {
            return (this.calcWidth * 20) / 100;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "pinnedWidth", {
        get: function () {
            return this.getPinnedWidth();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "unpinnedWidth", {
        get: function () {
            return this.getUnpinnedWidth();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "summariesMargin", {
        get: function () {
            return this.rowSelectable ? this.calcRowCheckboxWidth : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "columns", {
        get: function () {
            return this._columns;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "pinnedColumns", {
        get: function () {
            return this._pinnedColumns.filter(function (col) { return !col.hidden; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "unpinnedColumns", {
        get: function () {
            return this._unpinnedColumns.filter(function (col) { return !col.hidden; }).sort(function (col1, col2) { return col1.index - col2.index; });
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype.getColumnByName = function (name) {
        return this.columnList.find(function (col) { return col.field === name; });
    };
    IgxGridComponent.prototype.getRowByIndex = function (index) {
        return this.gridAPI.get_row_by_index(this.id, index);
    };
    IgxGridComponent.prototype.getRowByKey = function (keyValue) {
        return this.gridAPI.get_row_by_key(this.id, keyValue);
    };
    Object.defineProperty(IgxGridComponent.prototype, "visibleColumns", {
        get: function () {
            return this.columnList.filter(function (col) { return !col.hidden; });
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype.getCellByColumn = function (rowSelector, columnField) {
        return this.gridAPI.get_cell_by_field(this.id, rowSelector, columnField);
    };
    Object.defineProperty(IgxGridComponent.prototype, "totalPages", {
        get: function () {
            if (this.pagingState) {
                return this.pagingState.metadata.countPages;
            }
            return -1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "totalRecords", {
        get: function () {
            if (this.pagingState) {
                return this.pagingState.metadata.countRecords;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "isFirstPage", {
        get: function () {
            return this.page === 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "isLastPage", {
        get: function () {
            return this.page + 1 >= this.totalPages;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "totalWidth", {
        get: function () {
            var cols = this.visibleColumns;
            var totalWidth = 0;
            var i = 0;
            for (i; i < cols.length; i++) {
                totalWidth += parseInt(cols[i].width, 10) || 0;
            }
            return totalWidth;
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype.nextPage = function () {
        if (!this.isLastPage) {
            this.page += 1;
        }
    };
    IgxGridComponent.prototype.previousPage = function () {
        if (!this.isFirstPage) {
            this.page -= 1;
        }
    };
    IgxGridComponent.prototype.paginate = function (val) {
        if (val < 0) {
            return;
        }
        this.page = val;
    };
    IgxGridComponent.prototype.markForCheck = function () {
        if (this.rowList) {
            this.rowList.forEach(function (row) { return row.cdr.markForCheck(); });
        }
        this.cdr.detectChanges();
    };
    IgxGridComponent.prototype.addRow = function (data) {
        this.data.push(data);
        this.onRowAdded.emit({ data: data });
        this._pipeTrigger++;
        this.cdr.markForCheck();
    };
    IgxGridComponent.prototype.deleteRow = function (rowSelector) {
        var row = this.gridAPI.get_row_by_key(this.id, rowSelector);
        if (row) {
            var index = this.data.indexOf(row.rowData);
            if (this.rowSelectable === true) {
                this.deselectRows([row.rowID]);
            }
            this.data.splice(index, 1);
            this.onRowDeleted.emit({ data: row.rowData });
            this._pipeTrigger++;
            this.cdr.markForCheck();
        }
    };
    IgxGridComponent.prototype.updateCell = function (value, rowSelector, column) {
        var cell = this.gridAPI.get_cell_by_field(this.id, rowSelector, column);
        if (cell) {
            cell.update(value);
            this._pipeTrigger++;
        }
    };
    IgxGridComponent.prototype.updateRow = function (value, rowSelector) {
        var row = this.gridAPI.get_row_by_key(this.id, rowSelector);
        if (row) {
            if (this.primaryKey !== undefined && this.primaryKey !== null) {
                value[this.primaryKey] = row.rowData[this.primaryKey];
            }
            this.gridAPI.update_row(value, this.id, row);
            this._pipeTrigger++;
            this.cdr.markForCheck();
        }
    };
    IgxGridComponent.prototype.sort = function () {
        var rest = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            rest[_i] = arguments[_i];
        }
        if (rest.length === 1 && rest[0] instanceof Array) {
            this._sortMultiple(rest[0]);
        }
        else {
            this._sort(rest[0], rest[1], rest[2]);
        }
    };
    IgxGridComponent.prototype.filter = function () {
        var rest = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            rest[_i] = arguments[_i];
        }
        if (rest.length === 1 && rest[0] instanceof Array) {
            this._filterMultiple(rest[0]);
        }
        else {
            this._filter(rest[0], rest[1], rest[2], rest[3]);
        }
    };
    IgxGridComponent.prototype.filterGlobal = function (value, condition, ignoreCase) {
        this.gridAPI.filter_global(this.id, value, condition, ignoreCase);
    };
    IgxGridComponent.prototype.enableSummaries = function () {
        var rest = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            rest[_i] = arguments[_i];
        }
        if (rest.length === 1 && Array.isArray(rest[0])) {
            this._multipleSummaries(rest[0], true);
        }
        else {
            this._summaries(rest[0], true, rest[1]);
        }
        this.tfootHeight = 0;
        this.markForCheck();
        this.calculateGridHeight();
        this.cdr.detectChanges();
    };
    IgxGridComponent.prototype.disableSummaries = function () {
        var rest = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            rest[_i] = arguments[_i];
        }
        if (rest.length === 1 && Array.isArray(rest[0])) {
            this._disableMultipleSummaries(rest[0], false);
        }
        else {
            this._summaries(rest[0], false);
        }
        this.tfootHeight = 0;
        this.markForCheck();
        this.calculateGridHeight();
        this.cdr.detectChanges();
    };
    IgxGridComponent.prototype.clearFilter = function (name) {
        if (!name) {
            this.filteringExpressions = [];
            return;
        }
        if (!this.gridAPI.get_column_by_name(this.id, name)) {
            return;
        }
        this.clearSummaryCache();
        this.gridAPI.clear_filter(this.id, name);
    };
    IgxGridComponent.prototype.clearSort = function (name) {
        if (!name) {
            this.sortingExpressions = [];
            return;
        }
        if (!this.gridAPI.get_column_by_name(this.id, name)) {
            return;
        }
        this.gridAPI.clear_sort(this.id, name);
    };
    IgxGridComponent.prototype.clearSummaryCache = function () {
        this.gridAPI.remove_summary(this.id);
    };
    IgxGridComponent.prototype.pinColumn = function (columnName) {
        var col = this.getColumnByName(columnName);
        var colWidth = parseInt(col.width, 10);
        if (col.pinned) {
            return false;
        }
        /**
                 * If the column that we want to pin is bigger or equal than the unpinned area we should not pin it.
                 * It should be also unpinned before pinning, since changing left/right pin area doesn't affect unpinned area.
                 */
        if (this.getUnpinnedWidth(true) - colWidth < this.unpinnedAreaMinWidth) {
            return false;
        }
        col.pinned = true;
        var index = this._pinnedColumns.length;
        var args = { column: col, insertAtIndex: index };
        this.onColumnPinning.emit(args);
        // update grid collections.
        if (this._pinnedColumns.indexOf(col) === -1) {
            this._pinnedColumns.splice(args.insertAtIndex, 0, col);
            if (this._unpinnedColumns.indexOf(col) !== -1) {
                this._unpinnedColumns.splice(this._unpinnedColumns.indexOf(col), 1);
            }
        }
        this.markForCheck();
        return true;
    };
    IgxGridComponent.prototype.unpinColumn = function (columnName) {
        var col = this.getColumnByName(columnName);
        if (!col.pinned) {
            return false;
        }
        col.pinned = false;
        this._unpinnedColumns.splice(col.index, 0, col);
        if (this._pinnedColumns.indexOf(col) !== -1) {
            this._pinnedColumns.splice(this._pinnedColumns.indexOf(col), 1);
        }
        this.markForCheck();
        return true;
    };
    /**
     * Recalculates grid width/height dimensions. Should be run when changing DOM elements dimentions manually that affect the grid's size.
     */
    /**
         * Recalculates grid width/height dimensions. Should be run when changing DOM elements dimentions manually that affect the grid's size.
         */
    IgxGridComponent.prototype.reflow = /**
         * Recalculates grid width/height dimensions. Should be run when changing DOM elements dimentions manually that affect the grid's size.
         */
    function () {
        this.calculateGridSizes();
    };
    Object.defineProperty(IgxGridComponent.prototype, "hasSortableColumns", {
        get: function () {
            return this.columnList.some(function (col) { return col.sortable; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "hasEditableColumns", {
        get: function () {
            return this.columnList.some(function (col) { return col.editable; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "hasFilterableColumns", {
        get: function () {
            return this.columnList.some(function (col) { return col.filterable; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "hasSummarizedColumns", {
        get: function () {
            return this.columnList.some(function (col) { return col.hasSummary; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "selectedCells", {
        get: function () {
            if (this.rowList) {
                return this.rowList.map(function (row) { return row.cells.filter(function (cell) { return cell.selected; }); })
                    .reduce(function (a, b) { return a.concat(b); }, []);
            }
            return [];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "rowBasedHeight", {
        get: function () {
            if (this.data && this.data.length) {
                return this.data.length * this.rowHeight;
            }
            return 0;
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype._derivePossibleHeight = function () {
        if ((this._height && this._height.indexOf("%") === -1) || !this._height) {
            return;
        }
        if (!this.nativeElement.parentNode.clientHeight) {
            var viewPortHeight = screen.height;
            this._height = this.rowBasedHeight <= viewPortHeight ? null : viewPortHeight.toString();
        }
        else {
            var parentHeight = this.nativeElement.parentNode.getBoundingClientRect().height;
            this._height = this.rowBasedHeight <= parentHeight ? null : this._height;
        }
        this.calculateGridHeight();
        this.cdr.detectChanges();
    };
    IgxGridComponent.prototype._derivePossibleWidth = function () {
        if (!this.columnWidth) {
            this.columnWidth = this.getPossibleColumnWidth();
            this.initColumns(this.columnList);
        }
        this.calculateGridWidth();
    };
    IgxGridComponent.prototype.calculateGridHeight = function () {
        var computed = this.document.defaultView.getComputedStyle(this.nativeElement);
        if (!this._height) {
            this.calcHeight = null;
            if (this.hasSummarizedColumns && !this.tfootHeight) {
                this.tfootHeight = this.tfoot.nativeElement.firstElementChild ?
                    this.calcMaxSummaryHeight() : 0;
            }
            return;
        }
        if (this._height && this._height.indexOf("%") !== -1) {
            /*height in %*/
            var pagingHeight = 0;
            if (this.paging) {
                pagingHeight = this.paginator.nativeElement.firstElementChild ?
                    this.paginator.nativeElement.clientHeight : 0;
            }
            if (!this.tfootHeight) {
                this.tfootHeight = this.tfoot.nativeElement.firstElementChild ?
                    this.calcMaxSummaryHeight() : 0;
            }
            this.calcHeight = parseInt(computed.getPropertyValue("height"), 10) -
                this.theadRow.nativeElement.clientHeight -
                this.tfootHeight - pagingHeight -
                this.scr.nativeElement.clientHeight;
        }
        else {
            var pagingHeight = 0;
            if (this.paging) {
                pagingHeight = this.paginator.nativeElement.firstElementChild ?
                    this.paginator.nativeElement.clientHeight : 0;
            }
            if (!this.tfootHeight) {
                this.tfootHeight = this.tfoot.nativeElement.firstElementChild ?
                    this.calcMaxSummaryHeight() : 0;
            }
            this.calcHeight = parseInt(this._height, 10) -
                this.theadRow.nativeElement.getBoundingClientRect().height -
                this.tfootHeight - pagingHeight -
                this.scr.nativeElement.clientHeight;
        }
    };
    IgxGridComponent.prototype.getPossibleColumnWidth = function () {
        var computedWidth = parseInt(this.document.defaultView.getComputedStyle(this.nativeElement).getPropertyValue("width"), 10);
        var maxColumnWidth = Math.max.apply(Math, this.visibleColumns.map(function (col) { return parseInt(col.width, 10); })
            .filter(function (width) { return !isNaN(width); }));
        var sumExistingWidths = this.visibleColumns
            .filter(function (col) { return col.width !== null; })
            .reduce(function (prev, curr) { return prev + parseInt(curr.width, 10); }, 0);
        if (this.rowSelectable) {
            computedWidth -= this.headerCheckboxContainer.nativeElement.clientWidth;
        }
        var visibleColsWithNoWidth = this.visibleColumns.filter(function (col) { return col.width === null; });
        maxColumnWidth = !Number.isFinite(sumExistingWidths) ?
            Math.max(computedWidth / visibleColsWithNoWidth.length, MINIMUM_COLUMN_WIDTH) :
            Math.max((computedWidth - sumExistingWidths) / visibleColsWithNoWidth.length, MINIMUM_COLUMN_WIDTH);
        return maxColumnWidth.toString();
    };
    IgxGridComponent.prototype.calculateGridWidth = function () {
        var computed = this.document.defaultView.getComputedStyle(this.nativeElement);
        if (this._width && this._width.indexOf("%") !== -1) {
            /* width in %*/
            this.calcWidth = parseInt(computed.getPropertyValue("width"), 10);
            return;
        }
        this.calcWidth = parseInt(this._width, 10);
    };
    IgxGridComponent.prototype.calcMaxSummaryHeight = function () {
        var _this = this;
        var maxSummaryLength = 0;
        this.columnList.filter(function (col) { return col.hasSummary; }).forEach(function (column) {
            _this.gridAPI.set_summary_by_column_name(_this.id, column.field);
            var currentLength = _this.gridAPI.get_summaries(_this.id).get(column.field).length;
            if (maxSummaryLength < currentLength) {
                maxSummaryLength = currentLength;
            }
        });
        return maxSummaryLength * (this.tfoot.nativeElement.clientHeight ? this.tfoot.nativeElement.clientHeight : DEFAULT_SUMMARY_HEIGHT);
    };
    IgxGridComponent.prototype.calculateGridSizes = function () {
        this.calculateGridWidth();
        this.cdr.detectChanges();
        this.calculateGridHeight();
        if (this.rowSelectable) {
            this.calcRowCheckboxWidth = this.headerCheckboxContainer.nativeElement.clientWidth;
        }
        this.cdr.detectChanges();
    };
    /**
     * Gets calculated width of the start pinned area
     * @param takeHidden If we should take into account the hidden columns in the pinned area
     */
    /**
         * Gets calculated width of the start pinned area
         * @param takeHidden If we should take into account the hidden columns in the pinned area
         */
    IgxGridComponent.prototype.getPinnedWidth = /**
         * Gets calculated width of the start pinned area
         * @param takeHidden If we should take into account the hidden columns in the pinned area
         */
    function (takeHidden) {
        if (takeHidden === void 0) { takeHidden = false; }
        var fc = takeHidden ? this._pinnedColumns : this.pinnedColumns;
        var sum = 0;
        for (var _i = 0, fc_1 = fc; _i < fc_1.length; _i++) {
            var col = fc_1[_i];
            sum += parseInt(col.width, 10);
        }
        if (this.rowSelectable) {
            sum += this.calcRowCheckboxWidth;
        }
        return sum;
    };
    /**
     * Gets calculated width of the unpinned area
     * @param takeHidden If we should take into account the hidden columns in the pinned area
     */
    /**
         * Gets calculated width of the unpinned area
         * @param takeHidden If we should take into account the hidden columns in the pinned area
         */
    IgxGridComponent.prototype.getUnpinnedWidth = /**
         * Gets calculated width of the unpinned area
         * @param takeHidden If we should take into account the hidden columns in the pinned area
         */
    function (takeHidden) {
        if (takeHidden === void 0) { takeHidden = false; }
        var width = this._width && this._width.indexOf("%") !== -1 ?
            this.calcWidth : parseInt(this._width, 10);
        return width - this.getPinnedWidth(takeHidden);
    };
    IgxGridComponent.prototype._sort = function (name, direction, ignoreCase) {
        if (direction === void 0) { direction = SortingDirection.Asc; }
        if (ignoreCase === void 0) { ignoreCase = true; }
        this.gridAPI.sort(this.id, name, direction, ignoreCase);
    };
    IgxGridComponent.prototype._sortMultiple = function (expressions) {
        this.gridAPI.sort_multiple(this.id, expressions);
    };
    IgxGridComponent.prototype._filter = function (name, value, condition, ignoreCase) {
        var col = this.gridAPI.get_column_by_name(this.id, name);
        if (col) {
            this.gridAPI
                .filter(this.id, name, value, condition || col.filteringCondition, ignoreCase || col.filteringIgnoreCase);
        }
        else {
            this.gridAPI.filter(this.id, name, value, condition, ignoreCase);
        }
    };
    IgxGridComponent.prototype._filterMultiple = function (expressions) {
        this.gridAPI.filter_multiple(this.id, expressions);
    };
    IgxGridComponent.prototype._summaries = function (fieldName, hasSummary, summaryOperand) {
        var column = this.gridAPI.get_column_by_name(this.id, fieldName);
        column.hasSummary = hasSummary;
        if (summaryOperand) {
            column.summaries = summaryOperand;
        }
    };
    IgxGridComponent.prototype._multipleSummaries = function (expressions, hasSummary) {
        var _this = this;
        expressions.forEach(function (element) {
            _this._summaries(element.fieldName, hasSummary, element.customSummary);
        });
    };
    IgxGridComponent.prototype._disableMultipleSummaries = function (expressions, hasSummary) {
        var _this = this;
        expressions.forEach(function (column) { _this._summaries(column, false); });
    };
    IgxGridComponent.prototype.resolveDataTypes = function (rec) {
        if (typeof rec === "number") {
            return DataType.Number;
        }
        else if (typeof rec === "boolean") {
            return DataType.Boolean;
        }
        else if (typeof rec === "object" && rec instanceof Date) {
            return DataType.Date;
        }
        return DataType.String;
    };
    IgxGridComponent.prototype.autogenerateColumns = function () {
        var _this = this;
        var factory = this.resolver.resolveComponentFactory(IgxColumnComponent);
        var fields = Object.keys(this.data[0]);
        var columns = [];
        fields.forEach(function (field) {
            var ref = _this.viewRef.createComponent(factory);
            ref.instance.field = field;
            ref.instance.dataType = _this.resolveDataTypes(_this.data[0][field]);
            ref.changeDetectorRef.detectChanges();
            columns.push(ref.instance);
        });
        this.columnList.reset(columns);
    };
    IgxGridComponent.prototype.initColumns = function (collection, cb) {
        var _this = this;
        if (cb === void 0) { cb = null; }
        collection.forEach(function (column, index) {
            column.gridID = _this.id;
            column.index = index;
            if (!column.width) {
                column.width = _this.columnWidth;
            }
            if (cb) {
                cb(column);
            }
        });
        this._columns = this.columnList.toArray();
        this._pinnedColumns = this.columnList.filter(function (c) { return c.pinned; });
        this._unpinnedColumns = this.columnList.filter(function (c) { return !c.pinned; });
    };
    IgxGridComponent.prototype.setEventBusSubscription = function () {
        var _this = this;
        this.eventBus.pipe(debounceTime(DEBOUNCE_TIME), takeUntil(this.destroy$)).subscribe(function () { return _this.cdr.detectChanges(); });
    };
    IgxGridComponent.prototype.setVerticalScrollSubscription = function () {
        var _this = this;
        /*
                    Until the grid component is destroyed,
                    Take the first event and unsubscribe
                    then merge with an empty observable after DEBOUNCE_TIME,
                    re-subscribe and repeat the process
                */
        this.verticalScrollContainer.onChunkLoad.pipe(takeUntil(this.destroy$), take(1), merge(of({})), delay(DEBOUNCE_TIME), repeat()).subscribe(function () {
            if (_this.cellInEditMode) {
                _this.cellInEditMode.inEditMode = false;
            }
            _this.eventBus.next();
        });
    };
    IgxGridComponent.prototype.onHeaderCheckboxClick = function (event) {
        this.allRowsSelected = event.checked;
        var newSelection = event.checked ?
            this.filteredData ?
                this.selectionAPI.append_items(this.id, this.selectionAPI.get_all_ids(this._filteredData, this.primaryKey)) :
                this.selectionAPI.get_all_ids(this.data, this.primaryKey) :
            this.filteredData ?
                this.selectionAPI.subtract_items(this.id, this.selectionAPI.get_all_ids(this._filteredData, this.primaryKey)) :
                [];
        this.triggerRowSelectionChange(newSelection, null, event, event.checked);
        this.checkHeaderChecboxStatus(event.checked);
    };
    Object.defineProperty(IgxGridComponent.prototype, "headerCheckboxAriaLabel", {
        get: function () {
            return this._filteringExpressions.length > 0 ?
                this.headerCheckbox && this.headerCheckbox.checked ? "Deselect all filtered" : "Select all filtered" :
                this.headerCheckbox && this.headerCheckbox.checked ? "Deselect all" : "Select all";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "template", {
        get: function () {
            return this.emptyGridTemplate;
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype.checkHeaderChecboxStatus = function (headerStatus) {
        if (headerStatus === undefined) {
            this.allRowsSelected = this.selectionAPI.are_all_selected(this.id, this.data);
            if (this.headerCheckbox) {
                this.headerCheckbox.indeterminate = !this.allRowsSelected && !this.selectionAPI.are_none_selected(this.id);
            }
            this.cdr.markForCheck();
        }
        if (this.headerCheckbox) {
            this.headerCheckbox.checked = headerStatus !== undefined ? headerStatus : false;
        }
    };
    IgxGridComponent.prototype.filteredItemsStatus = function (componentID, filteredData, primaryKey) {
        var currSelection = this.selectionAPI.get_selection(componentID);
        var atLeastOneSelected = false;
        var notAllSelected = false;
        if (currSelection) {
            for (var _i = 0, _a = Object.keys(filteredData); _i < _a.length; _i++) {
                var key = _a[_i];
                var dataItem = primaryKey ? filteredData[key][primaryKey] : filteredData[key];
                if (currSelection.indexOf(dataItem) !== -1) {
                    atLeastOneSelected = true;
                    if (notAllSelected) {
                        return "indeterminate";
                    }
                }
                else {
                    notAllSelected = true;
                    if (atLeastOneSelected) {
                        return "indeterminate";
                    }
                }
            }
        }
        return atLeastOneSelected ? "allSelected" : "noneSelected";
    };
    IgxGridComponent.prototype.updateHeaderChecboxStatusOnFilter = function (data) {
        if (!data) {
            data = this.data;
        }
        switch (this.filteredItemsStatus(this.id, data)) {
            case "allSelected": {
                if (!this.allRowsSelected) {
                    this.allRowsSelected = true;
                }
                if (this.headerCheckbox.indeterminate) {
                    this.headerCheckbox.indeterminate = false;
                }
                break;
            }
            case "noneSelected": {
                if (this.allRowsSelected) {
                    this.allRowsSelected = false;
                }
                if (this.headerCheckbox.indeterminate) {
                    this.headerCheckbox.indeterminate = false;
                }
                break;
            }
            default: {
                if (!this.headerCheckbox.indeterminate) {
                    this.headerCheckbox.indeterminate = true;
                }
                if (this.allRowsSelected) {
                    this.allRowsSelected = false;
                }
                break;
            }
        }
    };
    IgxGridComponent.prototype.selectedRows = function () {
        return this.selectionAPI.get_selection(this.id);
    };
    IgxGridComponent.prototype.selectRows = function (rowIDs, clearCurrentSelection) {
        var newSelection = clearCurrentSelection ? rowIDs : this.selectionAPI.select_items(this.id, rowIDs);
        this.triggerRowSelectionChange(newSelection);
    };
    IgxGridComponent.prototype.deselectRows = function (rowIDs) {
        var newSelection = this.selectionAPI.deselect_items(this.id, rowIDs);
        this.triggerRowSelectionChange(newSelection);
    };
    IgxGridComponent.prototype.selectAllRows = function () {
        this.triggerRowSelectionChange(this.selectionAPI.get_all_ids(this.data, this.id));
    };
    IgxGridComponent.prototype.deselectAllRows = function () {
        this.triggerRowSelectionChange([]);
    };
    IgxGridComponent.prototype.triggerRowSelectionChange = function (newSelection, row, event, headerStatus) {
        var oldSelection = this.selectionAPI.get_selection(this.id);
        var args = { oldSelection: oldSelection, newSelection: newSelection, row: row, event: event };
        this.onRowSelectionChange.emit(args);
        this.selectionAPI.set_selection(this.id, args.newSelection);
        this.checkHeaderChecboxStatus(headerStatus);
    };
    IgxGridComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    preserveWhitespaces: false,
                    selector: "igx-grid",
                    template: "<ng-template #defaultPager let-api>         <button [disabled]=\"api.isFirstPage\" (click)=\"api.paginate(0)\" igxButton=\"icon\" igxRipple igxRippleCentered=\"true\">             <igx-icon fontSet=\"material\" name=\"first_page\"></igx-icon>         </button>         <button [disabled]=\"api.isFirstPage\" (click)=\"api.previousPage()\" igxButton=\"icon\" igxRipple igxRippleCentered=\"true\">             <igx-icon fontSet=\"material\" name=\"chevron_left\"></igx-icon>         </button>         <span>{{ api.page + 1 }} of {{ api.totalPages }}</span>         <button [disabled]=\"api.isLastPage\" (click)=\"api.nextPage()\" igxRipple igxRippleCentered=\"true\" igxButton=\"icon\">             <igx-icon fontSet=\"material\" name=\"chevron_right\"></igx-icon>         </button>         <button [disabled]=\"api.isLastPage\" (click)=\"api.paginate(api.totalPages - 1)\" igxButton=\"icon\" igxRipple igxRippleCentered=\"true\">             <igx-icon fontSet=\"material\" name=\"last_page\"></igx-icon>         </button>         <select style=\"margin-left: 1rem;\" (change)=\"api.perPage = $event.target.value\">             <option [value]=\"val\" [selected]=\"api.perPage == val\" *ngFor=\"let val of [5, 10, 15, 25, 50, 100, 500]\">{{ val }}</option>         </select> </ng-template>  <div class=\"igx-grid__thead\" role=\"rowgroup\" [style.width.px]='calcWidth' #theadRow>     <div class=\"igx-grid__tr\" role=\"row\">         <ng-container *ngIf=\"rowSelectable\">             <div class=\"igx-grid__cbx-selection\" #headerCheckboxContainer>                 <igx-checkbox [checked]=\"allRowsSelected\" (change)=\"onHeaderCheckboxClick($event)\" disableRipple=\"true\" [aria-label]=\"headerCheckboxAriaLabel\" #headerCheckbox></igx-checkbox>             </div>         </ng-container>         <ng-container *ngIf=\"pinnedColumns.length > 0\">             <igx-grid-header [gridID]=\"id\" *ngFor=\"let col of pinnedColumns\" [column]=\"col\" [style.min-width.px]=\"col.width\" [style.flex-basis.px]=\"col.width\"></igx-grid-header>         </ng-container>         <ng-template igxFor let-col [igxForOf]=\"unpinnedColumns\" [igxForScrollOrientation]=\"'horizontal'\" [igxForScrollContainer]=\"parentVirtDir\"             [igxForContainerSize]='unpinnedWidth' #headerContainer>             <igx-grid-header [gridID]=\"id\" [column]=\"col\" [style.min-width.px]=\"col.width\" [style.flex-basis.px]='col.width'></igx-grid-header>         </ng-template>     </div> </div>  <div class=\"igx-grid__tbody\" role=\"rowgroup\" [style.height.px]='calcHeight' [style.width.px]='calcWidth' #tbody>     <ng-template igxFor let-rowData [igxForOf]=\"data | gridFiltering:filteringExpressions:filteringLogic:id:pipeTrigger         | gridSort:sortingExpressions:id:pipeTrigger         | gridPaging:page:perPage:id:pipeTrigger\" let-rowIndex=\"index\" [igxForScrollOrientation]=\"'vertical'\"         [igxForContainerSize]='calcHeight' [igxForItemSize]=\"rowHeight\" #verticalScrollContainer (onChunkPreload)=\"dataLoading($event)\">         <igx-grid-row [gridID]=\"id\" [index]=\"rowIndex\" [rowData]=\"rowData\">         </igx-grid-row>     </ng-template>     <ng-container *ngIf=\"!!filteredData && filteredData.length === 0\">         <ng-container *ngTemplateOutlet=\"template\">         </ng-container>     </ng-container> </div>   <div class=\"igx-grid__tfoot\" role=\"rowgroup\" [style.width.px]='calcWidth' #tfoot>     <div *ngIf=\"hasSummarizedColumns\" class=\"igx-grid__tr\" [style.height.px]=\"tfootHeight\" [style.marginLeft.px]=\"summariesMargin\" role=\"row\">         <ng-container *ngIf=\"pinnedColumns.length > 0\">             <igx-grid-summary [gridID]=\"id\" *ngFor=\"let col of pinnedColumns\"  [column]=\"col\" [style.min-width.px]=\"col.width\" [style.flex-basis.px]='col.width'></igx-grid-summary>         </ng-container>         <ng-template igxFor let-col [igxForOf]=\"unpinnedColumns\" [igxForScrollOrientation]=\"'horizontal'\" [igxForScrollContainer]=\"parentVirtDir\" [igxForContainerSize]='unpinnedWidth' #summaryContainer>             <igx-grid-summary [gridID]=\"id\" [column]=\"col\" [style.min-width.px]=\"col.width\" [style.flex-basis.px]='col.width'></igx-grid-summary>         </ng-template>     </div> </div>  <div class=\"igx-grid__scroll\" [style.height]=\"'18px'\" #scr [hidden]=\"calcWidth - totalWidth >= 0\">     <div class=\"igx-grid__scroll-start\" [style.width.px]='pinnedWidth' [hidden]=\"pinnedWidth === 0\"></div>     <div class=\"igx-grid__scroll-main\" [style.width.px]='unpinnedWidth'>         <ng-template igxFor [igxForOf]='[]' #scrollContainer>         </ng-template>     </div> </div>  <div class=\"igx-paginator igx-grid-paginator\" *ngIf=\"paging\" #paginator>     <ng-container *ngTemplateOutlet=\"paginationTemplate ? paginationTemplate : defaultPager; context: { $implicit: this }\">     </ng-container> </div>  <ng-template #emptyGrid>     <p class=\"igx-grid--empty\">No records found.</p> </ng-template>"
                },] },
    ];
    /** @nocollapse */
    IgxGridComponent.ctorParameters = function () { return [
        { type: IgxGridAPIService, },
        { type: IgxSelectionAPIService, },
        { type: ElementRef, },
        { type: NgZone, },
        { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
        { type: ChangeDetectorRef, },
        { type: ComponentFactoryResolver, },
        { type: IterableDiffers, },
        { type: ViewContainerRef, },
    ]; };
    IgxGridComponent.propDecorators = {
        "data": [{ type: Input },],
        "autoGenerate": [{ type: Input },],
        "id": [{ type: HostBinding, args: ["attr.id",] }, { type: Input },],
        "filteringLogic": [{ type: Input },],
        "filteringExpressions": [{ type: Input },],
        "paging": [{ type: Input },],
        "page": [{ type: Input },],
        "perPage": [{ type: Input },],
        "paginationTemplate": [{ type: Input },],
        "rowSelectable": [{ type: Input },],
        "height": [{ type: HostBinding, args: ["style.height",] }, { type: Input },],
        "width": [{ type: HostBinding, args: ["style.width",] }, { type: Input },],
        "evenRowCSS": [{ type: Input },],
        "oddRowCSS": [{ type: Input },],
        "rowHeight": [{ type: Input },],
        "columnWidth": [{ type: Input },],
        "primaryKey": [{ type: Input },],
        "onCellClick": [{ type: Output },],
        "onSelection": [{ type: Output },],
        "onRowSelectionChange": [{ type: Output },],
        "onColumnPinning": [{ type: Output },],
        "onEditDone": [{ type: Output },],
        "onColumnInit": [{ type: Output },],
        "onSortingDone": [{ type: Output },],
        "onFilteringDone": [{ type: Output },],
        "onPagingDone": [{ type: Output },],
        "onRowAdded": [{ type: Output },],
        "onRowDeleted": [{ type: Output },],
        "onDataPreLoad": [{ type: Output },],
        "onColumnResized": [{ type: Output },],
        "onContextMenu": [{ type: Output },],
        "columnList": [{ type: ContentChildren, args: [IgxColumnComponent, { read: IgxColumnComponent },] },],
        "rowList": [{ type: ViewChildren, args: [IgxGridRowComponent, { read: IgxGridRowComponent },] },],
        "emptyGridTemplate": [{ type: ViewChild, args: ["emptyGrid", { read: TemplateRef },] },],
        "parentVirtDir": [{ type: ViewChild, args: ["scrollContainer", { read: IgxForOfDirective },] },],
        "verticalScrollContainer": [{ type: ViewChild, args: ["verticalScrollContainer", { read: IgxForOfDirective },] },],
        "scr": [{ type: ViewChild, args: ["scr", { read: ElementRef },] },],
        "paginator": [{ type: ViewChild, args: ["paginator", { read: ElementRef },] },],
        "headerContainer": [{ type: ViewChild, args: ["headerContainer", { read: IgxForOfDirective },] },],
        "headerCheckboxContainer": [{ type: ViewChild, args: ["headerCheckboxContainer",] },],
        "headerCheckbox": [{ type: ViewChild, args: ["headerCheckbox", { read: IgxCheckboxComponent },] },],
        "theadRow": [{ type: ViewChild, args: ["theadRow",] },],
        "tbody": [{ type: ViewChild, args: ["tbody",] },],
        "tfoot": [{ type: ViewChild, args: ["tfoot",] },],
        "tabindex": [{ type: HostBinding, args: ["attr.tabindex",] },],
        "hostClass": [{ type: HostBinding, args: ["attr.class",] },],
        "hostRole": [{ type: HostBinding, args: ["attr.role",] },],
        "sortingExpressions": [{ type: Input },],
    };
    return IgxGridComponent;
}());
export { IgxGridComponent };
//# sourceMappingURL=grid.component.js.map