import { Component, ContentChild, ElementRef, EventEmitter, HostBinding, Inject, Input, Optional, Output, Renderer, ViewChild } from "@angular/core";
import "rxjs/add/observable/fromEvent";
import "rxjs/add/observable/interval";
import "rxjs/add/operator/debounce";
import { Observable } from "rxjs/Observable";
import { IgxNavigationService } from "../core/navigation";
import { HammerGesturesManager } from "../core/touch";
import { IgxNavDrawerMiniTemplateDirective, IgxNavDrawerTemplateDirective } from "./navigation-drawer.directives";
/**
 * **Ignite UI for Angular Navigation Drawer** -
 * [Documentation](https://www.infragistics.com/products/ignite-ui-angular/angular/components/navdrawer.html)
 *
 * The Ignite UI Navigation Drawer is a collapsible side navigation container commonly used in combination with the Navbar.
 *
 * Example:
 * ```html
 * <igx-nav-drawer id="navigation" [isOpen]="true">
 *   <ng-template igxDrawer>
 *     <nav>
 *       <span igxDrawerItem [isHeader]="true">Email</span>
 *       <span igxDrawerItem igxRipple>Inbox</span>
 *       <span igxDrawerItem igxRipple>Deleted</span>
 *       <span igxDrawerItem igxRipple>Sent</span>
 *     </nav>
 *   </ng-template>
 * </igx-nav-drawer>
 * ```
 */
var NEXT_ID = 0;
var IgxNavigationDrawerComponent = /** @class */ (function () {
    function IgxNavigationDrawerComponent(elementRef, _state, 
    // private animate: AnimationBuilder, TODO
    renderer, _touchManager) {
        var _this = this;
        this.elementRef = elementRef;
        this._state = _state;
        this.renderer = renderer;
        this._touchManager = _touchManager;
        this.cssClass = "igx-nav-drawer";
        /** ID of the component */
        this.id = "igx-nav-drawer-" + NEXT_ID++;
        /**
             * Position of the Navigation Drawer. Can be "left"(default) or "right". Only has effect when not pinned.
             */
        this.position = "left";
        /**
             * Enables the use of touch gestures to manipulate the drawer - such as swipe/pan from edge to open,
             * swipe toggle and pan drag.
             */
        this.enableGestures = true;
        /** State of the drawer. */
        this.isOpen = false;
        /** When pinned the drawer is relatively positioned instead of sitting above content. May require additional layout styling. */
        this.pin = false;
        /**
             * Minimum device width required for automatic pin to be toggled.
             * Default is 1024, can be set to a falsy value to disable this behavior.
             */
        this.pinThreshold = 1024;
        /**
             * Width of the drawer in its open state. Defaults to "280px".
             */
        this.width = "280px";
        /**
             * Width of the drawer in its mini state. Defaults to 60px.
             */
        this.miniWidth = "60px";
        /** Pinned state change output for two-way binding  */
        this.pinChange = new EventEmitter(true);
        /** Event fired as the Navigation Drawer is about to open. */
        this.opening = new EventEmitter();
        /** Event fired when the Navigation Drawer has opened. */
        this.opened = new EventEmitter();
        /** Event fired as the Navigation Drawer is about to close. */
        this.closing = new EventEmitter();
        /** Event fired when the Navigation Drawer has closed. */
        this.closed = new EventEmitter();
        this._gesturesAttached = false;
        this._widthCache = { width: null, miniWidth: null };
        this.css = {
            drawer: "igx-nav-drawer__aside",
            mini: "igx-nav-drawer__aside--mini",
            overlay: "igx-nav-drawer__overlay",
            styleDummy: "igx-nav-drawer__style-dummy"
        };
        /** Pan animation properties */
        this._panning = false;
        this._maxEdgeZone = 50;
        this.checkPinThreshold = function (evt) {
            var windowWidth;
            if (_this.pinThreshold) {
                windowWidth = _this.getWindowWidth();
                if (!_this.pin && windowWidth >= _this.pinThreshold) {
                    _this.pin = true;
                    _this.pinChange.emit(true);
                }
                else if (_this.pin && windowWidth < _this.pinThreshold) {
                    _this.pin = false;
                    _this.pinChange.emit(false);
                }
            }
        };
        this.swipe = function (evt) {
            // TODO: Could also force input type: http://stackoverflow.com/a/27108052
            if (!_this.enableGestures || evt.pointerType !== "touch") {
                return;
            }
            // HammerJS swipe is horizontal-only by default, don't check deltaY
            var deltaX;
            var startPosition;
            if (_this.position === "right") {
                // when on the right use inverse of deltaX
                deltaX = -evt.deltaX;
                startPosition = _this.getWindowWidth() - (evt.center.x + evt.distance);
            }
            else {
                deltaX = evt.deltaX;
                startPosition = evt.center.x - evt.distance;
            }
            // only accept closing swipe (ignoring minEdgeZone) when the drawer is expanded:
            if ((_this.isOpen && deltaX < 0) ||
                // positive deltaX from the edge:
                (deltaX > 0 && startPosition < _this.maxEdgeZone)) {
                _this.toggle(true);
            }
        };
        this.panstart = function (evt) {
            // TODO: test code
            if (!_this.enableGestures || _this.pin || evt.pointerType !== "touch") {
                return;
            }
            var startPosition = _this.position === "right" ? _this.getWindowWidth() - (evt.center.x + evt.distance)
                : evt.center.x - evt.distance;
            // cache width during animation, flag to allow further handling
            if (_this.isOpen || (startPosition < _this.maxEdgeZone)) {
                _this._panning = true;
                _this._panStartWidth = _this.getExpectedWidth(!_this.isOpen);
                _this._panLimit = _this.getExpectedWidth(_this.isOpen);
                _this.renderer.setElementClass(_this.overlay, "panning", true);
                _this.renderer.setElementClass(_this.drawer, "panning", true);
            }
        };
        this.pan = function (evt) {
            // TODO: input.deltaX = prevDelta.x + (center.x - offset.x);
            // get actual delta (not total session one) from event?
            // pan WILL also fire after a full swipe, only resize on flag
            if (!_this._panning) {
                return;
            }
            var right = _this.position === "right";
            // when on the right use inverse of deltaX
            var deltaX = right ? -evt.deltaX : evt.deltaX;
            var visibleWidth;
            var newX;
            var percent;
            visibleWidth = _this._panStartWidth + deltaX;
            if (_this.isOpen && deltaX < 0) {
                // when visibleWidth hits limit - stop animating
                if (visibleWidth <= _this._panLimit) {
                    return;
                }
                if (_this.hasAnimateWidth) {
                    percent = (visibleWidth - _this._panLimit) / (_this._panStartWidth - _this._panLimit);
                    newX = visibleWidth;
                }
                else {
                    percent = visibleWidth / _this._panStartWidth;
                    newX = evt.deltaX;
                }
                _this.setXSize(newX, percent.toPrecision(2));
            }
            else if (!_this.isOpen && deltaX > 0) {
                // when visibleWidth hits limit - stop animating
                if (visibleWidth >= _this._panLimit) {
                    return;
                }
                if (_this.hasAnimateWidth) {
                    percent = (visibleWidth - _this._panStartWidth) / (_this._panLimit - _this._panStartWidth);
                    newX = visibleWidth;
                }
                else {
                    percent = visibleWidth / _this._panLimit;
                    newX = (_this._panLimit - visibleWidth) * (right ? 1 : -1);
                }
                _this.setXSize(newX, percent.toPrecision(2));
            }
        };
        this.panEnd = function (evt) {
            if (_this._panning) {
                var deltaX = _this.position === "right" ? -evt.deltaX : evt.deltaX;
                var visibleWidth = _this._panStartWidth + deltaX;
                _this.resetPan();
                // check if pan brought the drawer to 50%
                if (_this.isOpen && visibleWidth <= _this._panStartWidth / 2) {
                    _this.close(true);
                }
                else if (!_this.isOpen && visibleWidth >= _this._panLimit / 2) {
                    _this.open(true);
                }
                _this._panStartWidth = null;
            }
        };
        this.toggleOpenedEvent = function (evt, fireEvents) {
            _this.elementRef.nativeElement.removeEventListener("transitionend", _this.toggleOpenedEvent, false);
            _this.opened.emit();
        };
        this.toggleClosedEvent = function (evt) {
            _this.elementRef.nativeElement.removeEventListener("transitionend", _this.toggleClosedEvent, false);
            _this.closed.emit();
        };
    }
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "element", {
        /**
         * Returns nativeElement of the component.
         */
        get: /**
             * Returns nativeElement of the component.
             */
        function () {
            return this.elementRef.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "template", {
        get: function () {
            if (this.miniTemplate && !this.isOpen) {
                return this.miniTemplate.template;
            }
            else if (this.contentTemplate) {
                return this.contentTemplate.template;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "miniTemplate", {
        get: function () {
            return this._miniTemplate;
        },
        set: function (v) {
            if (!this.isOpen) {
                this.setDrawerWidth(v ? this.miniWidth : "");
            }
            this._miniTemplate = v;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "flexWidth", {
        get: function () {
            if (!this.pin) {
                return "0px";
            }
            if (this.isOpen) {
                return this.width;
            }
            if (this.miniTemplate && this.miniWidth) {
                return this.miniWidth;
            }
            return "0px";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "drawer", {
        get: function () {
            return this._drawer.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "overlay", {
        get: function () {
            return this._overlay.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "styleDummy", {
        get: function () {
            return this._styleDummy.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "hasAnimateWidth", {
        /**
         * Property to decide whether to change width or translate the drawer from pan gesture.
         */
        get: /**
             * Property to decide whether to change width or translate the drawer from pan gesture.
             */
        function () {
            return this.pin || !!this.miniTemplate;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "maxEdgeZone", {
        /**
         * Used for touch gestures (swipe and pan).
         * Defaults to 50 (in px) and is extended to at least 110% of the mini template width if available.
         * @protected set method
         */
        get: /**
             * Used for touch gestures (swipe and pan).
             * Defaults to 50 (in px) and is extended to at least 110% of the mini template width if available.
             * @protected set method
             */
        function () {
            return this._maxEdgeZone;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "expectedWidth", {
        /**
         * Get the Drawer width for specific state. Will attempt to evaluate requested state and cache.
         */
        get: /**
             * Get the Drawer width for specific state. Will attempt to evaluate requested state and cache.
             */
        function () {
            return this.getExpectedWidth(false);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "expectedMiniWidth", {
        /**
         * Get the Drawer mini width for specific state. Will attempt to evaluate requested state and cache.
         */
        get: /**
             * Get the Drawer mini width for specific state. Will attempt to evaluate requested state and cache.
             */
        function () {
            return this.getExpectedWidth(true);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "touchManager", {
        get: function () {
            return this._touchManager;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxNavigationDrawerComponent.prototype, "state", {
        /**
         * Exposes optional navigation service
         */
        get: /**
             * Exposes optional navigation service
             */
        function () {
            return this._state;
        },
        enumerable: true,
        configurable: true
    });
    IgxNavigationDrawerComponent.prototype.ngOnInit = function () {
        // DOM and @Input()-s initialized
        if (this._state) {
            this._state.add(this.id, this);
        }
        if (this.isOpen) {
            this.setDrawerWidth(this.width);
        }
    };
    IgxNavigationDrawerComponent.prototype.ngAfterContentInit = function () {
        // wait for template and ng-content to be ready
        this.updateEdgeZone();
        this.checkPinThreshold();
        // need to set height without absolute positioning
        this.ensureDrawerHeight();
        this.ensureEvents();
        // TODO: apply platform-safe Ruler from http://plnkr.co/edit/81nWDyreYMzkunihfRgX?p=preview
        // (https://github.com/angular/angular/issues/6515), blocked by https://github.com/angular/angular/issues/6904
    };
    IgxNavigationDrawerComponent.prototype.ngOnDestroy = function () {
        this._touchManager.destroy();
        if (this._state) {
            this._state.remove(this.id);
        }
        if (this._resizeObserver) {
            this._resizeObserver.unsubscribe();
        }
    };
    IgxNavigationDrawerComponent.prototype.ngOnChanges = function (changes) {
        // simple settings can come from attribute set (rather than binding), make sure boolean props are converted
        if (changes.enableGestures && changes.enableGestures.currentValue !== undefined) {
            this.enableGestures = !!(this.enableGestures && this.enableGestures.toString() === "true");
            this.ensureEvents();
        }
        if (changes.pin && changes.pin.currentValue !== undefined) {
            this.pin = !!(this.pin && this.pin.toString() === "true");
            this.ensureDrawerHeight();
            if (this.pin) {
                this._touchManager.destroy();
                this._gesturesAttached = false;
            }
            else {
                this.ensureEvents();
            }
        }
        if (changes.pinThreshold) {
            if (this.pinThreshold) {
                this.ensureEvents();
                this.checkPinThreshold();
            }
            else if (this._resizeObserver) {
                this._resizeObserver.unsubscribe();
                this._resizeObserver = null;
            }
        }
        if (changes.width && this.isOpen) {
            this.setDrawerWidth(changes.width.currentValue);
        }
        if (changes.miniWidth) {
            if (!this.isOpen) {
                this.setDrawerWidth(changes.miniWidth.currentValue);
            }
            this.updateEdgeZone();
        }
    };
    /**
     * Toggle the open state of the Navigation Drawer.
     * @param fireEvents Optional flag determining whether events should be fired or not.
     */
    /**
         * Toggle the open state of the Navigation Drawer.
         * @param fireEvents Optional flag determining whether events should be fired or not.
         */
    IgxNavigationDrawerComponent.prototype.toggle = /**
         * Toggle the open state of the Navigation Drawer.
         * @param fireEvents Optional flag determining whether events should be fired or not.
         */
    function (fireEvents) {
        if (this.isOpen) {
            this.close(fireEvents);
        }
        else {
            this.open(fireEvents);
        }
    };
    /**
     * Open the Navigation Drawer. Has no effect if already opened.
     * @param fireEvents Optional flag determining whether events should be fired or not.
     */
    /**
         * Open the Navigation Drawer. Has no effect if already opened.
         * @param fireEvents Optional flag determining whether events should be fired or not.
         */
    IgxNavigationDrawerComponent.prototype.open = /**
         * Open the Navigation Drawer. Has no effect if already opened.
         * @param fireEvents Optional flag determining whether events should be fired or not.
         */
    function (fireEvents) {
        if (this._panning) {
            this.resetPan();
        }
        if (this.isOpen) {
            return;
        }
        if (fireEvents) {
            this.opening.emit();
        }
        this.isOpen = true;
        // TODO: Switch to animate API when available
        // var animationCss = this.animate.css();
        //     animationCss
        //         .setStyles({'width':'50px'}, {'width':'400px'})
        //         .start(this.elementRef.nativeElement)
        //         .onComplete(() => animationCss.setToStyles({'width':'auto'}).start(this.elementRef.nativeElement));
        this.elementRef.nativeElement.addEventListener("transitionend", this.toggleOpenedEvent, false);
        this.setDrawerWidth(this.width);
    };
    /**
     * Close the Navigation Drawer. Has no effect if already closed.
     * @param fireEvents Optional flag determining whether events should be fired or not.
     */
    /**
         * Close the Navigation Drawer. Has no effect if already closed.
         * @param fireEvents Optional flag determining whether events should be fired or not.
         */
    IgxNavigationDrawerComponent.prototype.close = /**
         * Close the Navigation Drawer. Has no effect if already closed.
         * @param fireEvents Optional flag determining whether events should be fired or not.
         */
    function (fireEvents) {
        if (this._panning) {
            this.resetPan();
        }
        if (!this.isOpen) {
            return;
        }
        if (fireEvents) {
            this.closing.emit();
        }
        this.isOpen = false;
        this.setDrawerWidth(this.miniTemplate ? this.miniWidth : "");
        this.elementRef.nativeElement.addEventListener("transitionend", this.toggleClosedEvent, false);
    };
    IgxNavigationDrawerComponent.prototype.set_maxEdgeZone = function (value) {
        this._maxEdgeZone = value;
    };
    IgxNavigationDrawerComponent.prototype.ensureDrawerHeight = function () {
        if (this.pin) {
            // TODO: nested in content?
            // setElementStyle warning https://github.com/angular/angular/issues/6563
            this.renderer.setElementStyle(this.drawer, "height", window.innerHeight + "px");
        }
    };
    /**
     * Get the Drawer width for specific state. Will attempt to evaluate requested state and cache.
     * @param mini Request mini width instead
     */
    /**
         * Get the Drawer width for specific state. Will attempt to evaluate requested state and cache.
         * @param mini Request mini width instead
         */
    IgxNavigationDrawerComponent.prototype.getExpectedWidth = /**
         * Get the Drawer width for specific state. Will attempt to evaluate requested state and cache.
         * @param mini Request mini width instead
         */
    function (mini) {
        if (mini) {
            if (!this.miniTemplate) {
                return 0;
            }
            if (this.miniWidth) {
                return parseFloat(this.miniWidth);
            }
            else {
                // if (!this.isOpen) { // This WON'T work due to transition timings...
                //     return this.elementRef.nativeElement.children[1].offsetWidth;
                // } else {
                if (this._widthCache.miniWidth === null) {
                    // force class for width calc. TODO?
                    this.renderer.setElementClass(this.styleDummy, this.css.drawer, true);
                    this.renderer.setElementClass(this.styleDummy, this.css.mini, true);
                    this._widthCache.miniWidth = this.styleDummy.offsetWidth;
                    this.renderer.setElementClass(this.styleDummy, this.css.drawer, false);
                    this.renderer.setElementClass(this.styleDummy, this.css.mini, false);
                }
                return this._widthCache.miniWidth;
            }
        }
        else {
            if (this.width) {
                return parseFloat(this.width);
            }
            else {
                if (this._widthCache.width === null) {
                    // force class for width calc. TODO?
                    this.renderer.setElementClass(this.styleDummy, this.css.drawer, true);
                    this._widthCache.width = this.styleDummy.offsetWidth;
                    this.renderer.setElementClass(this.styleDummy, this.css.drawer, false);
                }
                return this._widthCache.width;
            }
        }
    };
    IgxNavigationDrawerComponent.prototype.getWindowWidth = function () {
        return (window.innerWidth > 0) ? window.innerWidth : screen.width;
    };
    /**
     * Sets the drawer width.
     * @param width Width to set, must be valid CSS size string.
     */
    /**
         * Sets the drawer width.
         * @param width Width to set, must be valid CSS size string.
         */
    IgxNavigationDrawerComponent.prototype.setDrawerWidth = /**
         * Sets the drawer width.
         * @param width Width to set, must be valid CSS size string.
         */
    function (width) {
        var _this = this;
        window.requestAnimationFrame(function () {
            if (_this.drawer) {
                _this.renderer.setElementStyle(_this.drawer, "width", width);
            }
        });
    };
    /**
     * Get current Drawer width.
     */
    /**
         * Get current Drawer width.
         */
    IgxNavigationDrawerComponent.prototype.getDrawerWidth = /**
         * Get current Drawer width.
         */
    function () {
        return this.drawer.offsetWidth;
    };
    IgxNavigationDrawerComponent.prototype.ensureEvents = function () {
        var _this = this;
        // set listeners for swipe/pan only if needed, but just once
        if (this.enableGestures && !this.pin && !this._gesturesAttached) {
            // Built-in manager handler(L20887) causes endless loop and max stack exception.
            // https://github.com/angular/angular/issues/6993
            // Use ours for now (until beta.10):
            // this.renderer.listen(document, "swipe", this.swipe);
            this._touchManager.addGlobalEventListener("document", "swipe", this.swipe);
            this._gesturesAttached = true;
            // this.renderer.listen(document, "panstart", this.panstart);
            // this.renderer.listen(document, "pan", this.pan);
            this._touchManager.addGlobalEventListener("document", "panstart", this.panstart);
            this._touchManager.addGlobalEventListener("document", "panmove", this.pan);
            this._touchManager.addGlobalEventListener("document", "panend", this.panEnd);
        }
        if (this.pinThreshold && !this._resizeObserver) {
            this._resizeObserver = Observable.fromEvent(window, "resize").debounce(function () { return Observable.interval(150); })
                .subscribe(function (value) { _this.checkPinThreshold(); });
        }
    };
    IgxNavigationDrawerComponent.prototype.updateEdgeZone = function () {
        var maxValue;
        if (this.miniTemplate) {
            maxValue = Math.max(this._maxEdgeZone, this.getExpectedWidth(true) * 1.1);
            this.set_maxEdgeZone(maxValue);
        }
    };
    IgxNavigationDrawerComponent.prototype.resetPan = function () {
        this._panning = false;
        /* styles fail to apply when set on parent due to extra attributes, prob ng bug */
        this.renderer.setElementClass(this.overlay, "panning", false);
        this.renderer.setElementClass(this.drawer, "panning", false);
        this.setXSize(0, "");
    };
    /**
     * Sets the absolute position or width in case the drawer doesn't change position.
     * @param x the number pixels to translate on the X axis or the width to set. 0 width will clear the style instead.
     * @param opacity optional value to apply to the overlay
     */
    /**
         * Sets the absolute position or width in case the drawer doesn't change position.
         * @param x the number pixels to translate on the X axis or the width to set. 0 width will clear the style instead.
         * @param opacity optional value to apply to the overlay
         */
    IgxNavigationDrawerComponent.prototype.setXSize = /**
         * Sets the absolute position or width in case the drawer doesn't change position.
         * @param x the number pixels to translate on the X axis or the width to set. 0 width will clear the style instead.
         * @param opacity optional value to apply to the overlay
         */
    function (x, opacity) {
        var _this = this;
        // Angular polyfills patches window.requestAnimationFrame, but switch to DomAdapter API (TODO)
        window.requestAnimationFrame(function () {
            if (_this.hasAnimateWidth) {
                _this.renderer.setElementStyle(_this.drawer, "width", x ? Math.abs(x) + "px" : "");
            }
            else {
                _this.renderer.setElementStyle(_this.drawer, "transform", x ? "translate3d(" + x + "px,0,0)" : "");
                _this.renderer.setElementStyle(_this.drawer, "-webkit-transform", x ? "translate3d(" + x + "px,0,0)" : "");
            }
            if (opacity !== undefined) {
                _this.renderer.setElementStyle(_this.overlay, "opacity", opacity);
            }
        });
    };
    IgxNavigationDrawerComponent.decorators = [
        { type: Component, args: [{
                    providers: [HammerGesturesManager],
                    selector: "igx-nav-drawer",
                    template: "<ng-template #defaultItemsTemplate>     <div igxDrawerItem [isHeader]=\"true\">Navigation Drawer</div>     <div igxDrawerItem> Star by adding</div>     <div igxDrawerItem> <code>&lt;ng-template igxDrawer&gt;</code> </div>     <div igxDrawerItem> And some items inside </div>     <div igxDrawerItem> Style with igxDrawerItem </div>     <div igxDrawerItem> and igxRipple directives</div> </ng-template>  <div [hidden]=\"pin\"     class=\"igx-nav-drawer__overlay\"     [class.igx-nav-drawer__overlay--hidden]=\"!isOpen\"     (click)=\"close(true)\" #overlay> </div> <aside role=\"navigation\"     class=\"igx-nav-drawer__aside\"     [class.igx-nav-drawer__aside--collapsed]=\"!miniTemplate && !isOpen\"     [class.igx-nav-drawer__aside--mini]=\"miniTemplate && !isOpen\"     [class.igx-nav-drawer__aside--pinned]=\"pin\"     [class.igx-nav-drawer__aside--right]=\"position == 'right'\" #aside>      <ng-container *ngTemplateOutlet=\"template || defaultItemsTemplate\"></ng-container> </aside> <div class=\"igx-nav-drawer__style-dummy\" #dummy></div>"
                },] },
    ];
    /** @nocollapse */
    IgxNavigationDrawerComponent.ctorParameters = function () { return [
        { type: ElementRef, decorators: [{ type: Inject, args: [ElementRef,] },] },
        { type: IgxNavigationService, decorators: [{ type: Optional },] },
        { type: Renderer, },
        { type: HammerGesturesManager, },
    ]; };
    IgxNavigationDrawerComponent.propDecorators = {
        "cssClass": [{ type: HostBinding, args: ["class",] },],
        "id": [{ type: HostBinding, args: ["attr.id",] }, { type: Input },],
        "position": [{ type: Input },],
        "enableGestures": [{ type: Input },],
        "isOpen": [{ type: Input },],
        "pin": [{ type: Input },],
        "pinThreshold": [{ type: Input },],
        "width": [{ type: Input },],
        "miniWidth": [{ type: Input },],
        "pinChange": [{ type: Output },],
        "opening": [{ type: Output },],
        "opened": [{ type: Output },],
        "closing": [{ type: Output },],
        "closed": [{ type: Output },],
        "miniTemplate": [{ type: ContentChild, args: [IgxNavDrawerMiniTemplateDirective, { read: IgxNavDrawerMiniTemplateDirective },] },],
        "contentTemplate": [{ type: ContentChild, args: [IgxNavDrawerTemplateDirective, { read: IgxNavDrawerTemplateDirective },] },],
        "flexWidth": [{ type: HostBinding, args: ["style.flexBasis",] },],
        "_drawer": [{ type: ViewChild, args: ["aside",] },],
        "_overlay": [{ type: ViewChild, args: ["overlay",] },],
        "_styleDummy": [{ type: ViewChild, args: ["dummy",] },],
    };
    return IgxNavigationDrawerComponent;
}());
export { IgxNavigationDrawerComponent };
//# sourceMappingURL=navigation-drawer.component.js.map