import { animate, state, style, transition, trigger } from "@angular/animations";
import { CommonModule } from "@angular/common";
import { Component, ElementRef, EventEmitter, HostBinding, Input, NgModule, Optional, Output } from "@angular/core";
import { IgxNavigationService } from "../core/navigation";
/**
 * **Ignite UI for Angular Toast** -
 * [Documentation](https://www.infragistics.com/products/ignite-ui-angular/angular/components/toast.html)
 *
 * The Ignite UI Toast provides information and warning messages that are non-interactive and cannot
 * be dismissed by the user. Toasts can be displayed at the bottom, middle, or top of the page.
 *
 * Example:
 * ```html
 * <button (click)="toast.show()">Show notification</button>
 * <igx-toast #toast
 *           message="Notification displayed"
 *           displayTime="1000">
 * </igx-toast>
 * ```
 */
var NEXT_ID = 0;
var IgxToastComponent = /** @class */ (function () {
    function IgxToastComponent(elementRef, navService) {
        this.elementRef = elementRef;
        this.navService = navService;
        this.CSS_CLASSES = {
            IGX_TOAST_BOTTOM: "igx-toast--bottom",
            IGX_TOAST_MIDDLE: "igx-toast--middle",
            IGX_TOAST_TOP: "igx-toast--top"
        };
        /**
             * Identifier of the component
             * @type {string}
             */
        this.id = "igx-toast-" + NEXT_ID++;
        /**
             * Event is thrown prior toast is shown
             * @type {EventEmitter}
             */
        this.onShowing = new EventEmitter();
        /**
             * Event is shown when toast is shown
             * @type {EventEmitter}
             */
        this.onShown = new EventEmitter();
        /**
             * Event is thrown prior toast hidden
             * @type {EventEmitter}
             */
        this.onHiding = new EventEmitter();
        /**
             * Event is thrown when toast hidden
             * @type {EventEmitter}
             */
        this.onHidden = new EventEmitter();
        this.role = "alert";
        /**
             * Sets if the IgxToast component will be hidden after shown
             * Default value is true
             * @type {number}
             */
        this.autoHide = true;
        /**
             * The duration of time span in ms which the IgxToast component will be visible
             * after it is being shown.
             * Default value is 4000
             * @type {number}
             */
        this.displayTime = 4000;
        /**
             * The IgxToast component visual state state
             * @type {boolean}
             */
        this.isVisible = false;
        /**
             * Specifies the position of the IgxToast component. Possible options are IgxToastPosition.Top,
             * IgxToastPosition.Middle, IgxToastPosition.Bottom
             * @type {IgxToastPosition}
             */
        this.position = IgxToastPosition.Bottom;
    }
    Object.defineProperty(IgxToastComponent.prototype, "element", {
        /**
         * Return the nativeElement of the component
         */
        get: /**
             * Return the nativeElement of the component
             */
        function () {
            return this.elementRef.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Shows the IgxToast component and hides it after some time span
     * if autoHide is enabled
     */
    /**
         * Shows the IgxToast component and hides it after some time span
         * if autoHide is enabled
         */
    IgxToastComponent.prototype.show = /**
         * Shows the IgxToast component and hides it after some time span
         * if autoHide is enabled
         */
    function () {
        var _this = this;
        clearInterval(this.timeoutId);
        this.onShowing.emit(this);
        this.isVisible = true;
        if (this.autoHide) {
            this.timeoutId = setTimeout(function () {
                _this.hide();
            }, this.displayTime);
        }
        this.onShown.emit(this);
    };
    /**
     * Hides the IgxToast component
     */
    /**
         * Hides the IgxToast component
         */
    IgxToastComponent.prototype.hide = /**
         * Hides the IgxToast component
         */
    function () {
        this.onHiding.emit(this);
        this.isVisible = false;
        this.onHidden.emit(this);
        clearInterval(this.timeoutId);
    };
    /**
     * Wraps @show() method due @IToggleView interface implementation.
     */
    /**
         * Wraps @show() method due @IToggleView interface implementation.
         */
    IgxToastComponent.prototype.open = /**
         * Wraps @show() method due @IToggleView interface implementation.
         */
    function () {
        this.show();
    };
    /**
     * Wraps @hide() method due @IToggleView interface implementation.
     */
    /**
         * Wraps @hide() method due @IToggleView interface implementation.
         */
    IgxToastComponent.prototype.close = /**
         * Wraps @hide() method due @IToggleView interface implementation.
         */
    function () {
        this.hide();
    };
    /**
     * Invoke @close() or @open() method depending on the state of the component.
     */
    /**
         * Invoke @close() or @open() method depending on the state of the component.
         */
    IgxToastComponent.prototype.toggle = /**
         * Invoke @close() or @open() method depending on the state of the component.
         */
    function () {
        this.isVisible ? this.close() : this.open();
    };
    IgxToastComponent.prototype.mapPositionToClassName = function () {
        if (this.position === IgxToastPosition.Top) {
            return this.CSS_CLASSES.IGX_TOAST_TOP;
        }
        if (this.position === IgxToastPosition.Middle) {
            return this.CSS_CLASSES.IGX_TOAST_MIDDLE;
        }
        if (this.position === IgxToastPosition.Bottom) {
            return this.CSS_CLASSES.IGX_TOAST_BOTTOM;
        }
    };
    IgxToastComponent.prototype.ngOnInit = function () {
        if (this.navService && this.id) {
            this.navService.add(this.id, this);
        }
    };
    IgxToastComponent.prototype.ngOnDestroy = function () {
        if (this.navService && this.id) {
            this.navService.remove(this.id);
        }
    };
    IgxToastComponent.decorators = [
        { type: Component, args: [{
                    animations: [
                        trigger("animate", [
                            state("show", style({
                                opacity: 1
                            })),
                            transition("* => show", animate(".20s ease")),
                            transition("show => *", animate(".40s ease-out"))
                        ])
                    ],
                    selector: "igx-toast",
                    template: "<div [ngClass]=\"mapPositionToClassName()\" *ngIf=\"this.isVisible\" [@animate]=\"'show'\">     {{ message }} </div>"
                },] },
    ];
    /** @nocollapse */
    IgxToastComponent.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: IgxNavigationService, decorators: [{ type: Optional },] },
    ]; };
    IgxToastComponent.propDecorators = {
        "id": [{ type: HostBinding, args: ["attr.id",] }, { type: Input },],
        "onShowing": [{ type: Output },],
        "onShown": [{ type: Output },],
        "onHiding": [{ type: Output },],
        "onHidden": [{ type: Output },],
        "role": [{ type: Input },],
        "autoHide": [{ type: Input },],
        "displayTime": [{ type: Input },],
        "isVisible": [{ type: Input },],
        "message": [{ type: Input },],
        "position": [{ type: Input },],
    };
    return IgxToastComponent;
}());
export { IgxToastComponent };
/**
 * Enumeration for toast position
 * Can be:
 * Bottom
 * Middle
 * Top
 */
/**
 * Enumeration for toast position
 * Can be:
 * Bottom
 * Middle
 * Top
 */
export var IgxToastPosition;
/**
 * Enumeration for toast position
 * Can be:
 * Bottom
 * Middle
 * Top
 */
(function (IgxToastPosition) {
    IgxToastPosition[IgxToastPosition["Bottom"] = 0] = "Bottom";
    IgxToastPosition[IgxToastPosition["Middle"] = 1] = "Middle";
    IgxToastPosition[IgxToastPosition["Top"] = 2] = "Top";
})(IgxToastPosition || (IgxToastPosition = {}));
var IgxToastModule = /** @class */ (function () {
    function IgxToastModule() {
    }
    IgxToastModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [IgxToastComponent],
                    exports: [IgxToastComponent],
                    imports: [CommonModule]
                },] },
    ];
    return IgxToastModule;
}());
export { IgxToastModule };
//# sourceMappingURL=toast.component.js.map